# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - EventAggregatorSummary Action

    @copyright: 2008, 2009, 2010, 2011 by Paul Boddie <paul@boddie.org.uk>
    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2003-2008 MoinMoin:ThomasWaldmann,
                2004-2006 MoinMoin:AlexanderSchremmer,
                2007 MoinMoin:ReimarBauer.
                2009 Cristian Rigamonti <rigamonti@fsfeurope.org>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin import config
from MoinMoin.Page import Page
from MoinMoin import wikiutil
from EventAggregatorSupport import *

Dependencies = ['pages']

# Action class and supporting functions.

class EventAggregatorSummary(ActionBase, ActionSupport):

    "A summary dialogue requesting various parameters."

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request
        form = self.get_form()

        resolution = form.get("resolution", ["month"])[0]

        category_list = []
        category_pagenames = form.get("category", [])

        for category_name, category_pagename in getCategoryMapping(getCategories(request), request):

            selected = self._get_selected_for_list(category_pagename, category_pagenames)

            category_list.append('<option value="%s" %s>%s</option>' % (
                escattr(category_pagename), selected, escape(category_name)))

        # Initialise month lists and defaults.

        start_month_list, end_month_list = self.get_month_lists()
        start_day_default, end_day_default = self.get_day_defaults()
        start_year_default, end_year_default = self.get_year_defaults()

        # Criteria instead of months and years.

        start_criteria_default = form.get("start", [""])[0]
        end_criteria_default = form.get("end", [""])[0]

        if resolution == "date":
            get_parameter = getParameterDate
            get_label = getFullDateLabel
        else:
            get_parameter = getParameterMonth
            get_label = getFullMonthLabel

        start_criteria_evaluated = get_parameter(start_criteria_default)
        end_criteria_evaluated = get_parameter(end_criteria_default)

        start_criteria_evaluated = get_label(request, start_criteria_evaluated)
        end_criteria_evaluated = get_label(request, end_criteria_evaluated)

        # Descriptions.

        descriptions = form.get("descriptions", [None])[0]

        descriptions_list = [
            '<option value="%s" %s>%s</option>' % ("page", self._get_selected("page", descriptions), escape(_("page"))),
            '<option value="%s" %s>%s</option>' % ("comment", self._get_selected("comment", descriptions), escape(_("comment")))
            ]

        # Format.

        format = form.get("format", [None])[0]

        format_list = [
            '<option value="%s" %s>%s</option>' % ("iCalendar", self._get_selected("iCalendar", format), escape(_("iCalendar"))),
            '<option value="%s" %s>%s</option>' % ("RSS", self._get_selected("RSS", format), escape(_("RSS 2.0")))
            ]

        right_arrow = unicode('\xe2\x86\x92', "utf-8")

        d = {
            "buttons_html"          : buttons_html,
            "category_label"        : escape(_("Categories")),
            "category_list"         : "\n".join(category_list),
            "start_month_list"      : "\n".join(start_month_list),
            "start_label"           : escape(_("Start day (optional), month and year")),
            "start_day_default"     : escattr(start_day_default),
            "start_year_default"    : escattr(start_year_default),
            "start_criteria_label"  : escape(_("or special criteria")),
            "start_criteria_default": escattr(start_criteria_default),
            "start_eval_label"      : escattr(right_arrow),
            "start_criteria_eval"   : escape(start_criteria_evaluated),
            "end_month_list"        : "\n".join(end_month_list),
            "end_label"             : escape(_("End day (optional), month and year")),
            "end_day_default"       : escattr(end_day_default),
            "end_year_default"      : escattr(end_year_default),
            "end_criteria_label"    : escape(_("or special criteria")),
            "end_criteria_default"  : escattr(end_criteria_default),
            "end_eval_label"        : escattr(right_arrow),
            "end_criteria_eval"     : escape(end_criteria_evaluated),
            "descriptions_label"    : escape(_("Use descriptions from...")),
            "descriptions_list"     : "\n".join(descriptions_list),
            "format_label"          : escape(_("Summary format")),
            "format_list"           : "\n".join(format_list),
            "parent_label"          : escape(_("Parent page")),
            "parent_name"           : escattr(form.get("parent", [""])[0]),
            "resolution"            : escattr(resolution),
            }

        return '''
<input name="resolution" type="hidden" value="%(resolution)s" />
<table>
    <tr>
        <td class="label"><label>%(category_label)s</label></td>
        <td class="content">
            <select multiple="multiple" name="category">
                %(category_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(start_label)s</label></td>
        <td>
            <input name="start-day" type="text" value="%(start_day_default)s" size="2" />
            <select name="start-month">
                %(start_month_list)s
            </select>
            <input name="start-year" type="text" value="%(start_year_default)s" size="4" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(start_criteria_label)s</label></td>
        <td>
            <input name="start" type="text" value="%(start_criteria_default)s" size="12" />
            <input name="start-eval" type="submit" value="%(start_eval_label)s" />
            %(start_criteria_eval)s
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(end_label)s</label></td>
        <td>
            <input name="end-day" type="text" value="%(end_day_default)s" size="2" />
            <select name="end-month">
                %(end_month_list)s
            </select>
            <input name="end-year" type="text" value="%(end_year_default)s" size="4" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(end_criteria_label)s</label></td>
        <td>
            <input name="end" type="text" value="%(end_criteria_default)s" size="12" />
            <input name="end-eval" type="submit" value="%(end_eval_label)s" />
            %(end_criteria_eval)s
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(descriptions_label)s</label></td>
        <td class="content">
            <select name="descriptions">
                %(descriptions_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(format_label)s</label></td>
        <td class="content">
            <select name="format">
                %(format_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(parent_label)s</label></td>
        <td class="content">
            <input name="parent" type="text" size="40" value="%(parent_name)s" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>
''' % d

    def do_action(self):

        "Write the iCalendar resource."

        _ = self._
        form = self.get_form()

        # If no category names exist in the request, an error message is
        # returned.

        category_names = form.get("category", [])

        if not category_names:
            return 0, _("No categories specified.")

        write_resource(self.request)
        return 1, None

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since a resource has already been
        produced.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

def getQuotedText(text):

    "Return the 'text' quoted for iCalendar purposes."

    return text.replace(";", r"\;").replace(",", r"\,")

def write_resource(request):

    """
    For the given 'request', write an iCalendar summary of the event data found
    in the categories specified via the "category" request parameter, using the
    "start" and "end" parameters (if specified). Multiple "category" parameters
    can be specified.
    """

    form = get_form(request)

    category_names = form.get("category", [])
    format = form.get("format", ["iCalendar"])[0]
    descriptions = form.get("descriptions", ["page"])[0]
    parent = form.get("parent", [""])[0]
    resolution = form.get("resolution", ["month"])[0]

    # Look first for a single start and end parameter. If that fails to provide
    # dates, look for separate start and end parameters, either for complete
    # dates or for years and months.

    if resolution == "date":
        calendar_start = getFormDate(request, None, "start")
        calendar_end = getFormDate(request, None, "end")

        if calendar_start is None:
            calendar_start = getFormDateTriple(request, "start-year", "start-month", "start-day")
        if calendar_end is None:
            calendar_end = getFormDateTriple(request, "end-year", "end-month", "end-day")

    elif resolution == "month":
        calendar_start = getFormMonth(request, None, "start")
        calendar_end = getFormMonth(request, None, "end")

        if calendar_start is None:
            calendar_start = getFormMonthPair(request, "start-year", "start-month")
        if calendar_end is None:
            calendar_end = getFormMonthPair(request, "end-year", "end-month")

    # Determine the period and get the events involved.

    event_pages = getPagesFromResults(getAllCategoryPages(category_names, request), request)
    events = getEventsFromPages(event_pages)
    all_shown_events = getEventsInPeriod(events, getCalendarPeriod(calendar_start, calendar_end))
    latest_timestamp = setEventTimestamps(request, all_shown_events)

    # Output summary data...

    if hasattr(request, "http_headers"):
        send_headers = request.http_headers
    elif hasattr(request, "emit_http_headers"):
        send_headers = request.emit_http_headers
    else:
        send_headers = send_headers(request)

    # Define headers.

    if format == "iCalendar":
        headers = ["Content-Type: text/calendar; charset=%s" % config.charset]
    elif format == "RSS":
        headers = ["Content-Type: application/rss+xml; charset=%s" % config.charset]

    # Define the last modified time.

    if latest_timestamp is not None:
        headers.append("Last-Modified: %s" % getHTTPTimeString(latest_timestamp))

    send_headers(headers)

    # iCalendar output...

    if format == "iCalendar":
        request.write("BEGIN:VCALENDAR\r\n")
        request.write("PRODID:-//MoinMoin//EventAggregatorSummary\r\n")
        request.write("VERSION:2.0\r\n")

        for event in all_shown_events:
            event_page = event.getPage()
            event_details = event.getDetails()

            # Get the summary details.

            event_summary = event.getSummary(parent)
            link = event_page.getPageURL(request)

            # Output the event details.

            request.write("BEGIN:VEVENT\r\n")
            request.write("UID:%s\r\n" % link)
            request.write("URL:%s\r\n" % link)
            request.write("DTSTAMP:%04d%02d%02dT%02d%02d%02dZ\r\n" % event_details["created"][:6])
            request.write("LAST-MODIFIED:%04d%02d%02dT%02d%02d%02dZ\r\n" % event_details["last-modified"][:6])
            request.write("SEQUENCE:%d\r\n" % event_details["sequence"])

            start = event_details["start"]
            end = event_details["end"]

            if isinstance(start, DateTime):
                request.write("DTSTART")
                write_calendar_datetime(request, start)
            else:
                request.write("DTSTART;VALUE=DATE:%04d%02d%02d\r\n" % start.as_date().as_tuple())

            if isinstance(end, DateTime):
                request.write("DTEND")
                write_calendar_datetime(request, end)
            else:
                request.write("DTEND;VALUE=DATE:%04d%02d%02d\r\n" % end.next_day().as_date().as_tuple())

            request.write("SUMMARY:%s\r\n" % getQuotedText(event_summary))

            # Optional details.

            if event_details.get("topics") or event_details.get("categories"):
                request.write("CATEGORIES:%s\r\n" % ",".join(
                    [getQuotedText(topic) for topic in event_details.get("topics") or event_details.get("categories")]
                    ))
            if event_details.has_key("location"):
                request.write("LOCATION:%s\r\n" % getQuotedText(event_details["location"]))

            request.write("END:VEVENT\r\n")

        request.write("END:VCALENDAR\r\n")

    # RSS output...

    elif format == "RSS":

        # Using the page name and the page URL in the title, link and
        # description.

        if hasattr(request, "getPathinfo"):
            path_info = request.getPathinfo()
        else:
            path_info = request.path

        request.write('<rss version="2.0">\r\n')
        request.write('<channel>\r\n')
        request.write('<title>%s</title>\r\n' % path_info[1:])
        request.write('<link>%s%s</link>\r\n' % (request.getBaseURL(), path_info))
        request.write('<description>Events published on %s%s</description>\r\n' % (request.getBaseURL(), path_info))

        if latest_timestamp is not None:
            request.write('<lastBuildDate>%s</lastBuildDate>\r\n' % getHTTPTimeString(latest_timestamp))
 
        # Sort all_shown_events by start date, reversed.

        ordered_events = getOrderedEvents(all_shown_events)
        ordered_events.reverse()

        for event in ordered_events:
            event_page = event.getPage()
            event_details = event.getDetails()

            # Get the summary details.

            event_summary = event.getSummary(parent)
            link = event_page.getPageURL(request)

            request.write('<item>\r\n')
            request.write('<title>%s</title>\r\n' % wikiutil.escape(event_summary))
            request.write('<link>%s</link>\r\n' % link)

            # Write a description according to the preferred source of
            # descriptions.

            if descriptions == "page":
                description = event_details.get("description", "")
            else:
                description = event_details["last-comment"]

            request.write('<description>%s</description>\r\n' % wikiutil.escape(description))

            for topic in event_details.get("topics") or event_details.get("categories") or []:
                request.write('<category>%s</category>\r\n' % topic)

            request.write('<pubDate>%s</pubDate>\r\n' % getHTTPTimeString(event_details["created"]))
            request.write('<guid>%s#%s</guid>\r\n' % (link, event_details["sequence"]))
            request.write('</item>\r\n')

        request.write('</channel>\r\n')
        request.write('</rss>\r\n')

def write_calendar_datetime(request, datetime):

    """
    Write to the given 'request' the 'datetime' using appropriate time zone
    information.
    """

    utc_datetime = datetime.to_utc()
    if utc_datetime:
        request.write(";VALUE=DATE-TIME:%04d%02d%02dT%02d%02d%02dZ\r\n" % utc_datetime.padded().as_tuple()[:-1])
    else:
        zone = datetime.time_zone()
        if zone:
            request.write(";TZID=/%s" % zone)
        request.write(";VALUE=DATE-TIME:%04d%02d%02dT%02d%02d%02d\r\n" % datetime.padded().as_tuple()[:-1])

# Action function.

def execute(pagename, request):
    EventAggregatorSummary(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
